@@ -7,6 +7,7 @@ import functools
 import inspect
 import threading
 import time
+import traceback
 import warnings
 import weakref
 from collections import deque
@@ -35,22 +36,6 @@ from typing import (
 
 from ..utils.inspect import ensure_coroutine
 
-__all__ = [
-    "AsyncEventIterator",
-    "AsyncEvent",
-    "async_event",
-    "AsyncTaskingEventIterator",
-    "AsyncTaskingEvent",
-    "async_tasking_event_iterator",
-    "async_tasking_event",
-    "check_canceled",
-    "run_in_thread",
-    "run_coroutine_in_thread",
-    "Lock",
-    "create_sub_task",
-    "FutureInfo",
-]
-
 _T = TypeVar("_T")
 
 _TResult = TypeVar("_TResult")
@@ -476,10 +461,11 @@ class Lock:
         try:
             try:
 
-                def aaa() -> None:
-                    warnings.warn(f"Lock takes to long {threading.current_thread()}")
+                def aaa(s: Any) -> None:
+                    warnings.warn(f"Lock takes to long {threading.current_thread()}\n{s}, try to cancel...")
+                    fut.cancel()
 
-                h = fut.get_loop().call_later(120, aaa)
+                h = fut.get_loop().call_later(120, aaa, "".join(traceback.format_stack()))
 
                 await fut
 
